
// Defininm variables globals
var keyring;
    //config;

// Definim un objecte amb tots els missatges per a tenir-los centralitzats
var missatges = {
  missatgeNoPgp : {id: 'missatgeNoPgp', type: 'error', text: 'No s\'ha trobat cap missatge OpenPGP',
    html: '<div class="alert alert-error" id="missatgeNoPgp">No s\'ha trobat cap missatge OpenPGP</div>' },
  missatgePasswordIncorrecte: {id: 'missatgePasswordIncorrecte', type: 'error', text: 'No s\'ha pogut llegir la clau privada',
    html: '<div class="alert alert-error" id="missatgePasswordIncorrecte">No s\'ha pogut llegir la clau privada</div>'},
  missatgeErrorDesxifrar: {id: 'missatgeErrorDesxifrar', type: 'error', text: 'No s\'ha pogut desxifrar el missatge',
    html: '<div class="alert alert-error" id="missatgeErrorDesxifrar">No s\'ha pogut desxifrar el missatge</div>'},
  missatgeNoClauPublica: {id: 'missatgeNoClauPublica', type: 'error', text: 'No s\'ha pogut trobar la clau pública per a tots els destinataris',
    html: '<div class="alert alert-error" id="missatgeNoClauPublica">No s\'ha pogut trobar les claus públiques per a tots els destinataris</div>'},
  missatgeNoClauPrivada: {id: 'missatgeNoClauPrivada', type: 'error', text: 'No s\'ha pogut trobar la clau privada de l\'usuari',
    html: '<div class="alert alert-error" id="missatgeNoClauPublica">No s\'ha pogut trobar la clau privada de l\'usuari</div>'}
};


function getClaus(keyIds, keyringSet) {
  var claus = [];
  
  keyIds.forEach(function(keyId){
    claus.push(keyringSet.getForId(keyId.toHex(), true));
  });
  return claus;
}

function getClausDestinataris(recipients, from) {
  // Definim array per emmagatzemar les claus
  var claus = [];
  // Per a cada destinatari, obtenim la seva clau pública
  _(recipients.email).each(function(email) {
    var clauPublica = keyring.publicKeys.getForAddress(email);
    if (!_.isEmpty(clauPublica)) {
	  // I l'emmagatzemem al nostre array de claus
      claus.push(clauPublica[0]);
    }
  });

  // Comprovem si no hem pogut recuperar les claus i si el nombre de destinataris
  // és diferent del nombre de claus recuperades
  if (_.isEmpty(claus) || (_.size(recipients.email) != _.size(claus))){
    return missatges.missatgeNoClauPublica;
  }

  return claus;
}

function xifrar(recipients, from, message, callback) {
  var promise;
  // Obtenim les claus públiques dels destinataris
  var publicKeys = getClausDestinataris(recipients, from);
  // Si no hem pogut obtenir les claus i la funció getClausDestinataris ens ha retornat
  // un missatge d'error
  if(publicKeys && publicKeys.type && publicKeys.type == "error") {
    // Assignem a la promesa el missatge d'error
	promise = Promise.resolve(publicKeys);
  }
  else {
	// Si hem pogut obtenir les claus, assignem a la promesa la funció per a xifrar el missatge
    promise = openpgp.encryptMessage(publicKeys, message);
  }
 
  // Passem el resultat a la funció callback
  promise.then(function(result) {
    callback(result);
  });
}

var resultatDesxifrat = function(decrypted, status, result, callback) {
  sortida = {};
  sortida.decrypted = decrypted;
  sortida.status = status;
  sortida.result = result;
  var promise = Promise.resolve(sortida);
  // Passem el resultat a la funció callback
  promise.then(function(result) {
    callback(result);
  });
};

function desxifrar(senderEmail, msg, password, callback) {
  var status = [];
  try{
	// Intentem llegir missatge OpenPGP
    msg = openpgp.message.readArmored(msg);
  }
  catch (e) {
	// Si no hi trobem un misstage OpenPGP ben format, missatge error
    status.push(missatges.missatgeNoPgp);
    resultatDesxifrat(false, status, undefined, callback);
  }
  // Obtenim Id de les claus
  var keyIds = msg.getEncryptionKeyIds();
  // Obtenim clau privada xifrada
  var privateKeys = _.compact(getClaus(keyIds, keyring.privateKeys));
  // Si no hem trobat cap clau privada, missatge error
  if (_.size(privateKeys) === 0) {
    status.push(missatges.missatgeNoClauPrivada);
    resultatDesxifrat(false, status, undefined, callback);
  }
  // Obtenim les claus públiques del remitent (en cas que hi hagi més d'una)
  var publicKeys = keyring.publicKeys.getForAddress(senderEmail);
  
  // Intentem desxifrar el missatge amb totes les claus privades de l'usuari
  for (var r = 0; r < privateKeys.length; r++){
    var key = privateKeys[r];
	// Intentem desencriptar la clau privada amb el password introduït
    if (!key.decryptKeyPacket(keyIds, password)) {
      // Si el password no és correcte, missatge error
      status.push(missatges.missatgePasswordIncorrecte);
    }
	
	// Desxifrem missatge
	var promise = openpgp.decryptMessage(key, msg);
    promise.then(function(result) {
	  resultatDesxifrat(true, status, result, callback);
    }).catch(function(exception) {
      status.push(missatges.missatgeErrorDesxifrar);
      resultatDesxifrat(false, status, undefined, callback);
    });
  }
}

// Afegim un listener per a les crides que fem des del content script
chrome.extension.onRequest.addListener(function(request, sender, callback){
    
	var result;

    // Llegim claus cada vegada perque han pogut canviar
    keyring.publicKeys.keys = keyring.storeHandler.loadPublic();
    keyring.privateKeys.keys = keyring.storeHandler.loadPrivate();
	
	// Comprovem l'acció que volem atendre
    if (request.method == "xifrar") {
      result = xifrar(request.recipients, request.from, request.message, callback);
    }
    else if (request.method == "desxifrar") {
      result = desxifrar(request.senderEmail, request.msg, request.password, callback);
    }
    else{
      throw new Error("Operació no soportada");
    }
});

document.onload = function() {
  // Iniciem l'anell de claus
  keyring = new openpgp.Keyring();
}();
